@@ -123,7 +123,8 @@ class hipsolver_local_handle
 public:
     hipsolver_local_handle()
     {
-        hipsolverCreate(&m_handle);
+        if(hipsolverCreate(&m_handle) != HIPSOLVER_STATUS_SUCCESS)
+            throw std::runtime_error("ERROR: Could not create hipsolverHandle_t");
     }
     ~hipsolver_local_handle()
     {
@@ -156,7 +157,8 @@ class hipsolverRf_local_handle
 public:
     hipsolverRf_local_handle()
     {
-        hipsolverRfCreate(&m_handle);
+        if(hipsolverRfCreate(&m_handle) != HIPSOLVER_STATUS_SUCCESS)
+            throw std::runtime_error("ERROR: Could not create hipsolverRfHandle_t");
     }
     ~hipsolverRf_local_handle()
     {
@@ -182,6 +184,40 @@ public:
     }
 };
 
+class hipsolverSp_local_handle
+{
+    hipsolverSpHandle_t m_handle;
+
+public:
+    hipsolverSp_local_handle()
+    {
+        if(hipsolverSpCreate(&m_handle) != HIPSOLVER_STATUS_SUCCESS)
+            throw std::runtime_error("ERROR: Could not create hipsolverSpHandle_t");
+    }
+    ~hipsolverSp_local_handle()
+    {
+        hipsolverSpDestroy(m_handle);
+    }
+
+    hipsolverSp_local_handle(const hipsolverSp_local_handle&) = delete;
+
+    hipsolverSp_local_handle(hipsolverSp_local_handle&&) = delete;
+
+    hipsolverSp_local_handle& operator=(const hipsolverSp_local_handle&) = delete;
+
+    hipsolverSp_local_handle& operator=(hipsolverSp_local_handle&&) = delete;
+
+    // Allow hipsolverSp_local_handle to be used anywhere hipsolverSpHandle_t is expected
+    operator hipsolverSpHandle_t&()
+    {
+        return m_handle;
+    }
+    operator const hipsolverSpHandle_t&() const
+    {
+        return m_handle;
+    }
+};
+
 /* ============================================================================================
  */
 /*! \brief  local gesvdj params which is automatically created and destroyed  */
@@ -192,7 +228,8 @@ class hipsolver_local_gesvdj_info
 public:
     hipsolver_local_gesvdj_info()
     {
-        hipsolverDnCreateGesvdjInfo(&m_info);
+        if(hipsolverDnCreateGesvdjInfo(&m_info) != HIPSOLVER_STATUS_SUCCESS)
+            throw std::runtime_error("ERROR: Could not create hipsolverGesvdjInfo_t");
     }
     ~hipsolver_local_gesvdj_info()
     {
@@ -228,7 +265,8 @@ class hipsolver_local_syevj_info
 public:
     hipsolver_local_syevj_info()
     {
-        hipsolverDnCreateSyevjInfo(&m_info);
+        if(hipsolverDnCreateSyevjInfo(&m_info) != HIPSOLVER_STATUS_SUCCESS)
+            throw std::runtime_error("ERROR: Could not create hipsolverSyevjInfo_t");
     }
     ~hipsolver_local_syevj_info()
     {
@@ -390,8 +428,7 @@ public:
 template <typename T>
 T random_generator()
 {
-    // return rand()/( (T)RAND_MAX + 1);
-    return T(rand() % 10 + 1);
+    return std::uniform_int_distribution<int>(1, 10)(hipsolver_rng);
 };
 
 // for hipsolverComplex, generate 2 floats
@@ -399,7 +436,8 @@ T random_generator()
 template <>
 inline hipsolverComplex random_generator<hipsolverComplex>()
 {
-    return hipsolverComplex(rand() % 10 + 1, rand() % 10 + 1);
+    return hipsolverComplex(float(std::uniform_int_distribution<int>(1, 10)(hipsolver_rng)),
+                            float(std::uniform_int_distribution<int>(1, 10)(hipsolver_rng)));
 }
 
 // for hipsolverDoubleComplex, generate 2 doubles
@@ -407,15 +445,15 @@ inline hipsolverComplex random_generator<hipsolverComplex>()
 template <>
 inline hipsolverDoubleComplex random_generator<hipsolverDoubleComplex>()
 {
-    return hipsolverDoubleComplex(rand() % 10 + 1, rand() % 10 + 1);
+    return hipsolverDoubleComplex(double(std::uniform_int_distribution<int>(1, 10)(hipsolver_rng)),
+                                  double(std::uniform_int_distribution<int>(1, 10)(hipsolver_rng)));
 }
 
 /*! \brief  generate a random number in range [-1,-2,-3,-4,-5,-6,-7,-8,-9,-10] */
 template <typename T>
 inline T random_generator_negative()
 {
-    // return rand()/( (T)RAND_MAX + 1);
-    return -T(rand() % 10 + 1);
+    return std::uniform_int_distribution<int>(-10, -1)(hipsolver_rng);
 };
 
 // for complex, generate two values, convert both to negative
@@ -425,13 +463,16 @@ inline T random_generator_negative()
 template <>
 inline hipsolverComplex random_generator_negative<hipsolverComplex>()
 {
-    return {float(-(rand() % 10 + 1)), float(-(rand() % 10 + 1))};
+    return hipsolverComplex(float(std::uniform_int_distribution<int>(-10, -1)(hipsolver_rng)),
+                            float(std::uniform_int_distribution<int>(-10, -1)(hipsolver_rng)));
 }
 
 template <>
 inline hipsolverDoubleComplex random_generator_negative<hipsolverDoubleComplex>()
 {
-    return {double(-(rand() % 10 + 1)), double(-(rand() % 10 + 1))};
+    return hipsolverDoubleComplex(
+        double(std::uniform_int_distribution<int>(-10, -1)(hipsolver_rng)),
+        double(std::uniform_int_distribution<int>(-10, -1)(hipsolver_rng)));
 }
 
 /* ============================================================================================ */
@@ -467,6 +508,27 @@ int type2int(T val);
 
 /* ============================================================================================ */
 /*! \brief  Debugging purpose, print out CPU and GPU result matrix, not valid in complex number  */
+template <typename T, std::enable_if_t<!is_complex<T>, int> = 0>
+void print_matrix(T* result, int m, int n, int lda)
+{
+    for(int i = 0; i < m; i++)
+        for(int j = 0; j < n; j++)
+            printf("matrix  col %d, row %d, result=%.8g\n", j, i, double(result[i + j * lda]));
+}
+
+/*! \brief  Debugging purpose, print out CPU and GPU result matrix, valid for complex number  */
+template <typename T, std::enable_if_t<+is_complex<T>, int> = 0>
+void print_matrix(T* result, int m, int n, int lda)
+{
+    for(int i = 0; i < m; i++)
+        for(int j = 0; j < n; j++)
+            printf("matrix  col %d, row %d, result=(%.8g,%.8g)\n",
+                   j,
+                   i,
+                   double(result[i + j * lda].real()),
+                   double(result[i + j * lda].imag()));
+}
+
 template <typename T, std::enable_if_t<!is_complex<T>, int> = 0>
 void print_matrix(T* CPU_result, T* GPU_result, int m, int n, int lda)
 {
@@ -494,6 +556,32 @@ void print_matrix(T* CPU_result, T* GPU_result, int m, int n, int lda)
                    double(GPU_result[i + j * lda].imag()));
 }
 
+/*! \brief  Debugging purpose, print out sparse matrix, not valid in complex number  */
+template <typename T, std::enable_if_t<!is_complex<T>, int> = 0>
+void print_sparse_matrix(int m, int n, int* csrRowPtr, int* csrColInd, T* csrVal)
+{
+    printf("%d by %d matrix\n", m, n);
+
+    int idx = 0;
+    for(int i = 0; i < m; i++)
+    {
+        printf("    ");
+        for(int j = 0; j < n; j++)
+        {
+            if(idx < csrRowPtr[i + 1] && j == csrColInd[idx])
+            {
+                printf("%.8g, ", csrVal[idx]);
+                idx++;
+            }
+            else
+                printf("0, ");
+        }
+        printf("\n");
+    }
+
+    printf("\n");
+}
+
 /* ============================================================================================ */
 /*  read matrix or values from file */
 
@@ -515,7 +603,9 @@ inline void
         for(int i = 0; i < m; ++i)
         {
             int read = fscanf(mat, "%d", &v);
-            assert(read == 1);
+            if(read != 1)
+                throw std::out_of_range(std::string("Error: Could not read element from file ")
+                                        + filename);
             A[i + j * lda] = v;
         }
     }
@@ -559,7 +649,9 @@ inline void
         for(int i = 0; i < m; ++i)
         {
             int read = fscanf(mat, "%g", &v);
-            assert(read == 1);
+            if(read != 1)
+                throw std::out_of_range(std::string("Error: Could not read element from file ")
+                                        + filename);
             A[i + j * lda] = v;
         }
     }
@@ -585,7 +677,9 @@ inline void
         for(int i = 0; i < m; ++i)
         {
             int read = fscanf(mat, "%lg", &v);
-            assert(read == 1);
+            if(read != 1)
+                throw std::out_of_range(std::string("Error: Could not read element from file ")
+                                        + filename);
             A[i + j * lda] = v;
         }
     }
